
/*

   (Colour) sum of bilinears in argument files.

*/

#include <stdio.h>
#include "spnhdr.h"

static char *usage        = "USAGE: colsum <file1> <file2> <file3>\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void sys_assert (condition, usr_message)
int condition ;
char *usr_message ;
/* Test `condition'.  Abort program, reporting `usr_message', if it fails.  */
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}


void get_params (nfiles, files, argc, argv)
int *nfiles ;
char ***files ;
int argc ;
char *argv [] ;
{
  int i ;

  if (argc < 2)
  {
    fprintf (stderr, usage) ;
    exit (1) ;
  }

  *nfiles = argc - 1 ;
  *files = (char **) malloc (*nfiles * sizeof (char *)) ;
  for (i = 0 ; i < *nfiles ; i++)
    (*files) [i] = argv [i + 1] ;
}

int merge_ints (vals, n)
int vals [] ;
int n ;
{
  int value ;
  int i ;

  value = vals [0] ;

  for (i = 0 ; i < n ; i++)
    if (vals [i] != value)
    {
      fprintf (stderr, "mismatch on int merge\n") ;
      exit (1) ;
    }

  return value ;
}

float merge_floats (vals, n)
float vals [] ;
int n ;
{
  float value ;
  int i ;

  value = vals [0] ;

  for (i = 0 ; i < n ; i++)
    if (vals [i] != value)
    {
      fprintf (stderr, "mismatch on float merge\n") ;
      exit (1) ;
    }

  return value ;
}

int merge_scs (vals, n)
/* Merge combined spin/colour indices, outputting simple spinor index */
int vals [3] ;
int n ;
{
  int value ;
  int i ;
  int got_col [3] ;

  if (n != 3)
  {
    fprintf (stderr, "expecting 3 colours\n") ;
    exit (1) ;
  }

  got_col [0] = 0 ;
  got_col [1] = 0 ;
  got_col [2] = 0 ;

  value = vals [0] / 6 ;

  for (i = 0 ; i < n ; i++)
  {
    int s = vals [i] / 6 ;
    int c = (vals [i] - (6 * s)) / 2 ;

    if ((vals [i] % 2) != 0)
    {
      fprintf (stderr, "spin/col index should be even\n") ;
      exit (1) ;
    }
    if (s != value)
    {
      fprintf (stderr, "spin mismatch on merge\n") ;
      exit (1) ;
    }
    if (got_col [c])
    {
      fprintf (stderr, "colours not different\n") ;
      exit (1) ;
    }
    else
      got_col [c] = 1 ;
  }

  return value ;
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  int nfiles, i ;
  char **files ;
  FILE **streams ;
  int   *s1s, *s2s, *l_times ;
  float *k1s, *k2s ;

  int l_time, t ;

  get_params  (&nfiles, &files, argc, argv) ;

  streams = (FILE **) malloc (nfiles * sizeof (FILE *)) ;
  for (i = 0 ; i < nfiles ; i++)
  {
    streams [i] = fopen (files [i], "r") ;
    sys_assert ((streams [i] != NULL), "Failed to open file") ;
  }

  l_times = (int *) malloc (nfiles * sizeof (int)) ;
  s1s = (  int *) malloc (nfiles * sizeof (  int)) ;
  k1s = (float *) malloc (nfiles * sizeof (float)) ;
  s2s = (  int *) malloc (nfiles * sizeof (  int)) ;
  k2s = (float *) malloc (nfiles * sizeof (float)) ;

  for (i = 0 ; i < nfiles ; i++)
  {
    /*
    fscanf (streams [i], "%d %d %g %d %g",
      l_times + i, s1s + i, k1s + i, s2s + i, k2s + i) ;
    */
    int result ;

    result = fread (l_times + i, sizeof (  int), 1, streams [i]) ;
    sys_assert (result == 1, "read failed") ;
    result = fread (s1s     + i, sizeof (  int), 1, streams [i]) ;
    sys_assert (result == 1, "read failed") ;
    result = fread (k1s     + i, sizeof (float), 1, streams [i]) ;
    sys_assert (result == 1, "read failed") ;
    result = fread (s2s     + i, sizeof (  int), 1, streams [i]) ;
    sys_assert (result == 1, "read failed") ;
    result = fread (k2s     + i, sizeof (float), 1, streams [i]) ;
    sys_assert (result == 1, "read failed") ;
  }

  l_time = merge_ints (l_times, nfiles) ;
  fprintf (stdout, "%d\n", l_time) ;
  fprintf (stdout, "%d\n", merge_scs    (s1s, nfiles)) ;
  fprintf (stdout, "%f\n", merge_floats (k1s, nfiles)) ;
  fprintf (stdout, "%d\n", merge_scs    (s2s, nfiles)) ;
  fprintf (stdout, "%f\n", merge_floats (k2s, nfiles)) ;
  fprintf (stdout, "\n") ;

  for (t = 0 ; t < l_time ; t++)
  {
    int s ;

    for (s = 0 ; s < N_BILS ; s++)
    {
      double acc_1, acc_2 ;

      acc_1 = 0.0 ;
      acc_2 = 0.0 ;
      for (i = 0 ; i < nfiles ; i++)
      {
	int result ;
	float term_1, term_2 ;

        /*
	fscanf (streams [i], "%g %g", &term_1, &term_2) ;
	*/
	result = fread (&term_1, sizeof (float), 1, streams [i]) ;
	sys_assert (result == 1, "read failed") ;
	result = fread (&term_2, sizeof (float), 1, streams [i]) ;
	sys_assert (result == 1, "read failed") ;

	acc_1 += term_1 ;
	acc_2 += term_2 ;
      }
      printf ("%g %g\n", acc_1, acc_2) ;
    }
    printf ("\n") ;
  }
}

